;;; -*- Mode: Emacs-Lisp -*-
;;; This mode is to tag articles for Named Entity, and Scenario Template.
;;; These emacs-lisp functions are created by Mr.Shinnou,
;;;  and modified by Mr.Sekine, and Mr.Nobata.
;;; To use this mode, please add the following description
;;;  to your .emacs file.
;;;
;;; (autoload 'pntag-mode "pntag-mode" nil t)
;;; (setq auto-mode-alist (append (list '("\\.key$" . pntag-mode))
;;;                               auto-mode-alist))

(defvar pntag-mode-map () "keymap used in pntag mode.")

(if pntag-mode-map
    ()
  (setq pntag-mode-map (make-sparse-keymap))
  (define-key pntag-mode-map "\C-x\C-x"    'pntag-organization)
  (define-key pntag-mode-map "\C-x\C-b"    'pntag-person)
  (define-key pntag-mode-map "\C-x\C-v"    'pntag-location)
  (define-key pntag-mode-map "\C-x\C-a"    'pntag-artifact)

  (define-key pntag-mode-map "\C-x\C-d"    'pntag-date)
  (define-key pntag-mode-map "\C-x\C-t"    'pntag-time)
  (define-key pntag-mode-map "\C-x\C-e"    'pntag-money)
  (define-key pntag-mode-map "\C-x\C-r"    'pntag-percent)

  (define-key pntag-mode-map "\C-x\C-q"    'pntag-uncertain)
  (define-key pntag-mode-map "\C-x\C-p"    'pntag-optional)
  (define-key pntag-mode-map "\C-x\C-o"    'pntag-optional-detail)
)

(defun pntag-mode ()
  "Major mode for tagging text."
  (interactive)
  (kill-all-local-variables)
  (use-local-map pntag-mode-map)
  (setq major-mode 'pntag-mode)
  (setq mode-name "pntag")
  (run-hooks 'pntag-mode-hook))

(defun pntag-organization  () (interactive) (pntag "ORGANIZATION"))
(defun pntag-person        () (interactive) (pntag "PERSON"))
(defun pntag-location      () (interactive) (pntag "LOCATION"))
(defun pntag-artifact      () (interactive) (pntag "ARTIFACT"))
(defun pntag-date          () (interactive) (pntag "DATE"))
(defun pntag-time          () (interactive) (pntag "TIME"))
(defun pntag-money         () (interactive) (pntag "MONEY"))
(defun pntag-percent       () (interactive) (pntag "PERCENT"))
(defun pntag-uncertain     () (interactive) (pntag "?"))
(defun pntag-optional      () (interactive) (pntag "OPTIONAL"))

(defun pntag-optional-detail () (interactive) 
  (let* ((pos  "")
	(type  ?0)
	(keys (read-string "Possibilities>"))
	(len  (length keys))
	(i    0))
    
    (while (< i len)
      (progn
	(setq key (elt keys i))
	(cond
	 ((= key ?x) (setq pos (concat pos "ORGANIZATION,")))
	 ((= key ?b) (setq pos (concat pos "PERSON,")))
	 ((= key ?v) (setq pos (concat pos "LOCATION,")))
	 ((= key ?a) (setq pos (concat pos "ARTIFACT,")))
	 ((= key ?d) (setq pos (concat pos "DATE,")))
	 ((= key ?t) (setq pos (concat pos "TIME,")))
	 ((= key ?e) (setq pos (concat pos "MONEY,")))
	 ((= key ?r) (setq pos (concat pos "PERCENT,")))
	 ((= key ?n) (setq pos (concat pos "NONE,")))
	 ((and (<= ?0 key) (<= key ?9)) (setq type key)))
      (setq i (+ 1 i))))

    (pntag2 (concat "OPTIONAL POSSIBILITY=" 
		    (if (= (length pos) 0) pos
		      (substring pos 0 (- (length pos) 1)) )
		    " TYPE=" 
		    (- type ?0))
	    "OPTIONAL")))

(defun pntag (str)
  (interactive)
  (let ((name))
      (if (null (mark))
          (error "No mark set in this buffer")
          (setq name (buffer-substring (mark) (point))))
      (delete-region (mark) (point))
      (goto-char (mark))
      (insert (concat "<" str ">" name "</" str ">"))
  )
)

(defun pntag2 (str1 str2)
  (interactive)
  (let ((name))
      (if (null (mark))
          (error "No mark set in this buffer")
          (setq name (buffer-substring (mark) (point))))
      (delete-region (mark) (point))
      (goto-char (mark))
      (insert (concat "<" str1 ">" name "</" str2 ">"))
  )
)
